from __future__ import annotations

from qgis.core import (
	QgsGeometry,
	QgsLineString,
	QgsMultiLineString,
	QgsMultiPoint,
	QgsMultiPolygon,
	QgsPoint,
	QgsPolygon,
	QgsPolyhedralSurface,
	QgsTriangle,
	QgsTriangulatedSurface,
)

Point = tuple[float, ...]
Cell = list[Point, ...]
Triangle = tuple[Point, Point, Point]


def coords2Points(points: list[Point]) -> list[QgsGeometry]:
	return [QgsPoint(*coords) for coords in points]


def coords2MultiPoint(points: list[Point]) -> QgsGeometry:
	return QgsMultiPoint([QgsPoint(*coords) for coords in points])


def coords2LineString(points: list[Point]) -> QgsGeometry:
	return QgsLineString(coords2Points(points))


def coords2MultiLineString(lines: list[Cell]) -> QgsGeometry:
	return QgsMultiLineString([coords2LineString(p) for p in lines])


def coords2Polygon(boundary: Cell) -> QgsGeometry:
	return QgsPolygon(QgsLineString(coords2Points(boundary)))


def coords2MultiPolygon(polygons: list[Cell]) -> QgsGeometry:
	return QgsMultiPolygon([coords2Polygon(p) for p in polygons])


def coords2PolyhedralSurface(polygons: list[Cell]) -> QgsGeometry:
	return QgsPolyhedralSurface(coords2MultiPolygon(polygons))


def coords2TriangulatedSurface(triangles: list[Triangle]) -> QgsGeometry:
	geom = QgsTriangulatedSurface()
	geom.setTriangles([QgsTriangle(tri) for tri in triangles])
	return geom
